
use v6;
module Perl::Class-0.0.1;

=pod

=head1 NAME

Perl::Class - A meta-model for Perl Classes

=head1 DESCRIPTION

This module will eventually hold the meta-model for perl6 classes.

=head1 AUTHORS

Sam Vilain

Stevan Little E<lt>stevan@iinteractive.comE<gt>

=cut
