#nav {
  width: 100%; 
}

#nav .link {
  width: 8em;
}

#nav .name {
  text-align: center;
  font-size: 150%;
}

#help {
  float:right;
}

td {
  vertical-align: top;
  padding: 0px;  
  margin: 0px;
#  border: 1px dotted #bb95a1;
}

td.src {
  padding: 0 50px 1em 25px;  
  #background: #e4f0fb;
}

.out {
  padding: 0px 2px 0px 2px;  
  #background: #e4f0fb;
  background: #fffde5;
  border: 1px dotted #bb95a1;
}

.para {
  display:none;
#  display:inline;
}

pre {
  padding: 0;  
  margin: 0;
}
