#!/usr/bin/pugs

use v6;
use Test;

plan 9;

use_ok('Hack::Instances');

#use_ok('Perl::MetaModel');

use_ok('Perl::MetaClass');
use_ok('Perl::MetaProperty');
use_ok('Perl::MetaMethod');
use_ok('Perl::MetaAssoc');

use_ok('Perl::Package');
use_ok('Perl::Module');
use_ok('Perl::Class');
use_ok('Perl::Role');
